@@ -60,7 +60,7 @@ pub use self::cookie::{Cookie, CookieJar};
 pub use self::tempfile::TempFile;
 use crate::{
     body::Body,
-    error::{Error, Result},
+    error::{Error, ReadBodyError, Result},
     http::{
         header::{HeaderMap, HeaderName},
         HeaderValue, Method, StatusCode, Uri, Version,
@@ -79,10 +79,8 @@ impl RequestBody {
 
     /// Take a body, if it has already been taken, an error with the status code
     /// [`StatusCode::INTERNAL_SERVER_ERROR`] is returned.
-    pub fn take(&mut self) -> Result<Body> {
-        self.0
-            .take()
-            .ok_or_else(|| Error::internal_server_error("the body has been taken"))
+    pub fn take(&mut self) -> Result<Body, ReadBodyError> {
+        self.0.take().ok_or(ReadBodyError::BodyHasBeenTaken)
     }
 }
 
@@ -206,18 +204,45 @@ impl RequestBody {
 /// token from the `MyToken` header.
 ///
 /// ```
+/// use std::{
+///     error::Error as StdError,
+///     fmt::{self, Display, Formatter},
+/// };
+///
 /// use poem::{handler, route, Endpoint, Error, FromRequest, Request, RequestBody};
 ///
 /// struct Token(String);
 ///
+/// #[derive(Debug)]
+/// struct MissingToken;
+///
+/// impl Display for MissingToken {
+///     fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+///         write!(f, "missing token")
+///     }
+/// }
+///
+/// impl StdError for MissingToken {}
+///
+/// impl From<MissingToken> for Error {
+///     fn from(err: MissingToken) -> Self {
+///         Error::bad_request(err)
+///     }
+/// }
+///
 /// #[poem::async_trait]
 /// impl<'a> FromRequest<'a> for Token {
-///     async fn from_request(req: &'a Request, body: &mut RequestBody) -> poem::Result<Self> {
+///     type Error = MissingToken;
+///
+///     async fn from_request(
+///         req: &'a Request,
+///         body: &mut RequestBody,
+///     ) -> Result<Self, Self::Error> {
 ///         let token = req
 ///             .headers()
 ///             .get("MyToken")
 ///             .and_then(|value| value.to_str().ok())
-///             .ok_or_else(|| Error::bad_request("missing token"))?;
+///             .ok_or(MissingToken)?;
 ///         Ok(Token(token.to_string()))
 ///     }
 /// }
@@ -239,8 +264,13 @@ impl RequestBody {
 
 #[async_trait::async_trait]
 pub trait FromRequest<'a>: Sized {
+    /// The error type of this extractor.
+    ///
+    /// If you don't know what type you should use, you can use [`Error`].
+    type Error: Into<Error>;
+
     /// Perform the extraction.
-    async fn from_request(req: &'a Request, body: &mut RequestBody) -> Result<Self>;
+    async fn from_request(req: &'a Request, body: &mut RequestBody) -> Result<Self, Self::Error>;
 }
 
 /// Represents a type that can convert into response.
@@ -538,11 +568,15 @@ impl<T: IntoResponse> IntoResponse for (StatusCode, HeaderMap, T) {
     }
 }
 
-impl<T: IntoResponse> IntoResponse for Result<T> {
+impl<T, E> IntoResponse for core::result::Result<T, E>
+where
+    T: IntoResponse,
+    E: IntoResponse,
+{
     fn into_response(self) -> Response {
         match self {
             Ok(resp) => resp.into_response(),
-            Err(err) => err.as_response(),
+            Err(err) => err.into_response(),
         }
     }
 }
@@ -560,105 +594,123 @@ impl<T: Into<String> + Send> IntoResponse for Html<T> {
 
 #[async_trait::async_trait]
 impl<'a> FromRequest<'a> for &'a Request {
-    async fn from_request(req: &'a Request, _body: &mut RequestBody) -> Result<Self> {
+    type Error = Infallible;
+
+    async fn from_request(req: &'a Request, _body: &mut RequestBody) -> Result<Self, Self::Error> {
         Ok(req)
     }
 }
 
 #[async_trait::async_trait]
 impl<'a> FromRequest<'a> for &'a Uri {
-    async fn from_request(req: &'a Request, _body: &mut RequestBody) -> Result<Self> {
+    type Error = Infallible;
+
+    async fn from_request(req: &'a Request, _body: &mut RequestBody) -> Result<Self, Self::Error> {
         Ok(req.uri())
     }
 }
 
 #[async_trait::async_trait]
 impl<'a> FromRequest<'a> for Method {
-    async fn from_request(req: &'a Request, _body: &mut RequestBody) -> Result<Self> {
+    type Error = Infallible;
+
+    async fn from_request(req: &'a Request, _body: &mut RequestBody) -> Result<Self, Self::Error> {
         Ok(req.method().clone())
     }
 }
 
 #[async_trait::async_trait]
 impl<'a> FromRequest<'a> for Version {
-    async fn from_request(req: &'a Request, _body: &mut RequestBody) -> Result<Self> {
+    type Error = Infallible;
+
+    async fn from_request(req: &'a Request, _body: &mut RequestBody) -> Result<Self, Self::Error> {
         Ok(req.version())
     }
 }
 
 #[async_trait::async_trait]
 impl<'a> FromRequest<'a> for &'a HeaderMap {
-    async fn from_request(req: &'a Request, _body: &mut RequestBody) -> Result<Self> {
+    type Error = Infallible;
+
+    async fn from_request(req: &'a Request, _body: &mut RequestBody) -> Result<Self, Self::Error> {
         Ok(req.headers())
     }
 }
 
 #[async_trait::async_trait]
 impl<'a> FromRequest<'a> for Body {
-    async fn from_request(_req: &'a Request, body: &mut RequestBody) -> Result<Self> {
+    type Error = ReadBodyError;
+
+    async fn from_request(_req: &'a Request, body: &mut RequestBody) -> Result<Self, Self::Error> {
         body.take()
     }
 }
 
 #[async_trait::async_trait]
 impl<'a> FromRequest<'a> for String {
-    async fn from_request(_req: &'a Request, body: &mut RequestBody) -> Result<Self> {
-        let data = body
-            .take()?
-            .into_bytes()
-            .await
-            .map_err(Error::bad_request)?;
-        String::from_utf8(data.to_vec()).map_err(Error::bad_request)
+    type Error = ReadBodyError;
+
+    async fn from_request(_req: &'a Request, body: &mut RequestBody) -> Result<Self, Self::Error> {
+        let data = body.take()?.into_bytes().await.map_err(ReadBodyError::Io)?;
+        String::from_utf8(data.to_vec()).map_err(ReadBodyError::Utf8)
     }
 }
 
 #[async_trait::async_trait]
 impl<'a> FromRequest<'a> for Bytes {
-    async fn from_request(_req: &'a Request, body: &mut RequestBody) -> Result<Self> {
-        Ok(body
-            .take()?
-            .into_bytes()
-            .await
-            .map_err(Error::bad_request)?)
+    type Error = ReadBodyError;
+
+    async fn from_request(_req: &'a Request, body: &mut RequestBody) -> Result<Self, Self::Error> {
+        Ok(body.take()?.into_bytes().await.map_err(ReadBodyError::Io)?)
     }
 }
 
 #[async_trait::async_trait]
 impl<'a> FromRequest<'a> for Vec<u8> {
-    async fn from_request(_req: &'a Request, body: &mut RequestBody) -> Result<Self> {
+    type Error = ReadBodyError;
+
+    async fn from_request(_req: &'a Request, body: &mut RequestBody) -> Result<Self, Self::Error> {
         Ok(body
             .take()?
             .into_bytes()
             .await
-            .map_err(Error::bad_request)?
+            .map_err(ReadBodyError::Io)?
             .to_vec())
     }
 }
 
 #[async_trait::async_trait]
 impl<'a> FromRequest<'a> for SocketAddr {
-    async fn from_request(req: &'a Request, _body: &mut RequestBody) -> Result<Self> {
+    type Error = Infallible;
+
+    async fn from_request(req: &'a Request, _body: &mut RequestBody) -> Result<Self, Self::Error> {
         Ok(req.state().remote_addr)
     }
 }
 
 #[async_trait::async_trait]
 impl<'a> FromRequest<'a> for IpAddr {
-    async fn from_request(req: &'a Request, _body: &mut RequestBody) -> Result<Self> {
+    type Error = Infallible;
+
+    async fn from_request(req: &'a Request, _body: &mut RequestBody) -> Result<Self, Self::Error> {
         Ok(req.state().remote_addr.ip())
     }
 }
 
 #[async_trait::async_trait]
 impl<'a, T: FromRequest<'a>> FromRequest<'a> for Option<T> {
-    async fn from_request(req: &'a Request, body: &mut RequestBody) -> Result<Self> {
+    type Error = T::Error;
+
+    async fn from_request(req: &'a Request, body: &mut RequestBody) -> Result<Self, Self::Error> {
         Ok(T::from_request(req, body).await.ok())
     }
 }
 
 #[async_trait::async_trait]
-impl<'a, T: FromRequest<'a>> FromRequest<'a> for Result<T> {
-    async fn from_request(req: &'a Request, body: &mut RequestBody) -> Result<Self> {
+impl<'a, T: FromRequest<'a>> FromRequest<'a> for Result<T, T::Error> {
+    type Error = Infallible;
+
+    async fn from_request(req: &'a Request, body: &mut RequestBody) -> Result<Self, Self::Error> {
         Ok(T::from_request(req, body).await)
     }
 }
