@@ -2,7 +2,7 @@ use std::ops::{Deref, DerefMut};
 
 use serde::de::DeserializeOwned;
 
-use crate::{Error, FromRequest, Request, RequestBody, Result};
+use crate::{error::ParseQueryError, FromRequest, Request, RequestBody, Result};
 
 /// An extractor that can deserialize some type from query string.
 pub struct Query<T>(pub T);
@@ -23,10 +23,10 @@ impl<T> DerefMut for Query<T> {
 
 #[async_trait::async_trait]
 impl<'a, T: DeserializeOwned> FromRequest<'a> for Query<T> {
-    async fn from_request(req: &'a Request, _body: &mut RequestBody) -> Result<Self> {
-        serde_urlencoded::from_str(req.uri().query().unwrap_or_default())
-            .map_err(Error::bad_request)
-            .map(Self)
+    type Error = ParseQueryError;
+
+    async fn from_request(req: &'a Request, _body: &mut RequestBody) -> Result<Self, Self::Error> {
+        Ok(serde_urlencoded::from_str(req.uri().query().unwrap_or_default()).map(Self)?)
     }
 }
 
